#!/usr/bin/perl
use strict;
use warnings;
use AutoDiff;
use Math::Matrix;

sub Jacobian {
  my $rf = shift;
  my $v = shift;
  my $dv = [ map {AutoDiff->new($_, 0)} @{$v}];
  my @j;
  foreach my $i (0..$#$v) {
    $dv->[$i]->set_der(1);
    push @j, [map {$_->[1]} @{$rf->($dv)}];
    $dv->[$i]->set_der(0);
  }
  Math::Matrix->new(@j)->transpose;
}

sub f {                              # function R^3 -> R
  my ($x, $y, $z) = @{$_[0]};
  return [$x**2+2*$y**2-$z**2];
}
my $v = [1, 1, 1];

print "Example 1:\n";
print Jacobian(\&f, $v),"\n";


sub g {                              # function R -> R^3
  my ($x) = @{$_[0]};
  return [$x,
          $x**2,
          $x**3];
}
$v = [1];

print "Example 2:\n";
print Jacobian(\&g, $v),"\n";

sub h {                              # function R^3 -> R^3
  my ($x, $y, $z) = @{$_[0]};
  return [ $x**2+$y**2+$z**2-1,
           2*$x**2+$y**2-4*$z,
           3*$x**2-4*$y+$z**2
         ];
}
$v = [1, 1, 1];

print "Example 3:\n";
print Jacobian(\&h, $v),"\n";
